#include <X11/Xlib.h>
#include "xc.h"
#include "wpw.bm"
#include "wpb.bm"
#include "bpw.bm"
#include "bpb.bm"
#include "wrw.bm"
#include "wrb.bm"
#include "brw.bm"
#include "brb.bm"
#include "wkw.bm"
#include "wkb.bm"
#include "bkw.bm"
#include "bkb.bm"
#include "wqw.bm"
#include "wqb.bm"
#include "bqw.bm"
#include "bqb.bm"
#include "wbw.bm"
#include "wbb.bm"
#include "bbw.bm"
#include "bbb.bm"
#include "wnw.bm"
#include "wnb.bm"
#include "bnw.bm"
#include "bnb.bm"
#include "white.bm"
#include "black.bm"

#define WIDTH SQUARESIZE
#define HEIGHT SQUARESIZE

#define BLACK (i % 2 && !(j % 2)) || (!(i % 2) && (j % 2))

#define create_pixmap(bits) \
        XCreateBitmapFromData(theDisplay, DefaultRootWindow(theDisplay), \
                              bits, WIDTH, HEIGHT)

#define copy_plane(pixmap) \
        XCopyPlane(theDisplay, pixmap, theWindow, \
                   theGC, 0, 0, WIDTH, HEIGHT, WIDTH*i, HEIGHT*j, 1);

#define draw_piece(black_square, white_square) \
        if (BLACK) { copy_plane(black_square); } else \
                   { copy_plane(white_square); }

draw_board(refresh)
Bool refresh;          /* if True, will redraw the entire board */
{
    int i, j;
    static Pixmap wpw_pixmap, wpb_pixmap, bpw_pixmap, bpb_pixmap;
    static Pixmap wrw_pixmap, wrb_pixmap, brw_pixmap, brb_pixmap;
    static Pixmap wkw_pixmap, wkb_pixmap, bkw_pixmap, bkb_pixmap;
    static Pixmap wqw_pixmap, wqb_pixmap, bqw_pixmap, bqb_pixmap;
    static Pixmap wbw_pixmap, wbb_pixmap, bbw_pixmap, bbb_pixmap;
    static Pixmap wnw_pixmap, wnb_pixmap, bnw_pixmap, bnb_pixmap;
    static Pixmap white_pixmap, black_pixmap;

    static int first = 1;

    if (first)
    {
        wpb_pixmap = create_pixmap(wpb_bits);
        wpw_pixmap = create_pixmap(wpw_bits);
        wbb_pixmap = create_pixmap(wbb_bits);
        wbw_pixmap = create_pixmap(wbw_bits);
        wkb_pixmap = create_pixmap(wkb_bits);
        wkw_pixmap = create_pixmap(wkw_bits);
        wqb_pixmap = create_pixmap(wqb_bits);
        wqw_pixmap = create_pixmap(wqw_bits);
        wnb_pixmap = create_pixmap(wnb_bits);
        wnw_pixmap = create_pixmap(wnw_bits);
        wrb_pixmap = create_pixmap(wrb_bits);
        wrw_pixmap = create_pixmap(wrw_bits);

        bpb_pixmap = create_pixmap(bpb_bits);
        bpw_pixmap = create_pixmap(bpw_bits);
        bbb_pixmap = create_pixmap(bbb_bits);
        bbw_pixmap = create_pixmap(bbw_bits);
        bkb_pixmap = create_pixmap(bkb_bits);
        bkw_pixmap = create_pixmap(bkw_bits);
        bqb_pixmap = create_pixmap(bqb_bits);
        bqw_pixmap = create_pixmap(bqw_bits);
        bnb_pixmap = create_pixmap(bnb_bits);
        bnw_pixmap = create_pixmap(bnw_bits);
        brb_pixmap = create_pixmap(brb_bits);
        brw_pixmap = create_pixmap(brw_bits);

        white_pixmap = create_pixmap(white_bits);
        black_pixmap = create_pixmap(black_bits);

        first = 0;
    }

    for (i=0; i<8; i++)
    {
        for (j=0; j<8; j++)
        {
            if ((old_squares[i][j] != squares[i][j]) || refresh)
            {
                XClearArea(theDisplay, theWindow, WIDTH*i, HEIGHT*j,
                           WIDTH, HEIGHT, False);

                switch(squares[i][j])
                {
                  case 'P' :
                    /* WHITE PAWN */

                    draw_piece(wpb_pixmap, wpw_pixmap);
                    break;

                  case 'p' :
                    /* BLACK PAWN */

                    draw_piece(bpb_pixmap, bpw_pixmap);
                    break;

                  case 'R' :
                    /* WHITE ROOK */

                    draw_piece(wrb_pixmap, wrw_pixmap);
                    break;

                  case 'r' :
                    /* BLACK ROOK */

                    draw_piece(brb_pixmap, brw_pixmap);
                    break;

                  case 'K' :
                    /* WHITE KING */

                    draw_piece(wkb_pixmap, wkw_pixmap);
                    break;

                  case 'k' :
                    /* BLACK KING */

                    draw_piece(bkb_pixmap, bkw_pixmap);
                    break;

                  case 'Q' :
                    /* WHITE QUEEN */

                    draw_piece(wqb_pixmap, wqw_pixmap);
                    break;

                  case 'q' :
                    /* BLACK QUEEN */

                    draw_piece(bqb_pixmap, bqw_pixmap);
                    break;

                  case 'N' :
                    /* WHITE KNIGHT */

                    draw_piece(wnb_pixmap, wnw_pixmap);
                    break;

                  case 'n' :
                    /* BLACK KNIGHT */

                    draw_piece(bnb_pixmap, bnw_pixmap);
                    break;

                  case 'B' :
                    /* WHITE BISHOP */

                    draw_piece(wbb_pixmap, wbw_pixmap);
                    break;

                  case 'b' :
                    /* BLACK BISHOP */

                    draw_piece(bbb_pixmap, bbw_pixmap);
                    break;

                  case ' ' :
                    /* BLANK SQUARE */

                    draw_piece(black_pixmap, white_pixmap);
                    break;
                }
            }
        }
    }
    /* Draw a line at the bottom */

    XDrawLine(theDisplay, theWindow, theGC, 0, WindowSize, WindowSize,
              WindowSize);

    /*  clear the text window  */

    XClearArea(theDisplay, theWindow, 0, WindowSize + 1, WindowSize,
               Statusheight, False);

    /* draw the text */

    for (i=0; i<8; i++)
    {
        XDrawString(theDisplay, theWindow, theGC,
                    10, WindowSize + 15*(i+2),
                    &text[i][0], strlen(&text[i][0]));
    }

    /* draw the buttons */

    for (i=0; i<NBUTTONS; i++)
    {
        XDrawLine(theDisplay, theWindow, theGC, buttons[i].x, buttons[i].y,
                  buttons[i].x + buttons[i].width, buttons[i].y);
        XDrawLine(theDisplay, theWindow, theGC, buttons[i].x, buttons[i].y,
                  buttons[i].x, buttons[i].y + buttons[i].height);
        XDrawLine(theDisplay, theWindow, theGC, buttons[i].x + buttons[i].width,
                  buttons[i].y, buttons[i].x + buttons[i].width,
                  buttons[i].y + buttons[i].height);
        XDrawLine(theDisplay, theWindow, theGC, buttons[i].x,
                  buttons[i].y + buttons[i].height,
                  buttons[i].x + buttons[i].width,
                  buttons[i].y + buttons[i].height);
        XDrawString(theDisplay, theWindow, theGC,
                    buttons[i].x, buttons[i].y + buttons[i].height/2 + 4,
                    buttons[i].command, strlen(buttons[i].command));
    }

    XFlush(theDisplay);
}
