#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/time.h>

#ifdef _AIX
#include <sys/select.h>
#endif

#include <X11/Xlib.h>                  
#include "xc.h"                      

#define syserr(string) (perror(string), exit(1))
#define fatal(string) (printf("ERROR : %s\n", string), exit(1))

extern void exit();

/*
 * nelson@reed.edu hacks: changed from non-blocking IO and busy loop to
 * call on select for various inputs, and the X server.
 *
 * Wed May 13 16:39:37 PDT 1992
 */

void telnet(host, portnum)
char * host;
char * portnum;
{
    int pfdout[2], pfdin[2], nbytes;
    char buf[512];
    char move[80];
    int done = 0;
    fd_set fileSet;                  /* nelson@reed.edu */
    fd_set * readFDs = &fileSet;              /* used for select */
    int xServerFD = ConnectionNumber(theDisplay);      /* watch here, too! */
    int nds;
    int childpid;
    int rc;
    struct timeval timeout; /*PDS*/
        
    timeout.tv_sec = 1; timeout.tv_usec = 0;
    
    if (pipe(pfdout) == -1 || pipe(pfdin) == -1) syserr("pipe");
  
    childpid = fork();
    switch(childpid)
    {
      case -1:
        syserr("fork");
        break;
    
      case 0:
        if (close(0) == -1) syserr("close");
        if (dup(pfdout[0]) != 0) fatal("dup");
        if (close(1) == -1) syserr("close");
        if (dup(pfdin[1]) != 1) fatal("dup");
        if (close(pfdout[0]) == -1 || close(pfdout[1]) == -1 ||
            close(pfdin[0]) == -1 || close(pfdin[1]) == -1)
        {
            syserr("close");
        }
    
        execlp("telnet", "telnet", host, portnum, NULL);
        syserr("execlp");
    }

    if (close(pfdout[0]) == -1 || close(pfdin[1]) == -1) syserr("close");

    /* main read loop for the program. Look for things from the keyboard,
     * from telnet, or from the X server.
     * nelson@reed.edu - replace original non blocking IO with regular
     * blocking IO, by calling select() to allow for multiple input sources.
     * this is much more efficient and fixes bugs. 
     */

    nds = (pfdin[0] > xServerFD ? pfdin[0] : xServerFD) + 1;

    while (!done)
    {
        checkclock();
       
        FD_ZERO(readFDs);                /* nelson@reed.edu */
        FD_SET(0, readFDs);              /* we care about these 3 */
        FD_SET(pfdin[0], readFDs);       /* file descriptors */
        FD_SET(xServerFD, readFDs);      /* kbd, telnet, and X */

#ifdef __hpux
        rc = select(nds, (int *)readFDs, 0, 0, &timeout);    /* wait for input */
#else
        rc = select(nds, readFDs, 0, 0, &timeout);    /* wait for input */
#endif
        if (rc < 0) syserr("select");

        if (FD_ISSET(0, readFDs))
        {
            nbytes = read(0, buf, sizeof(buf));
            if (nbytes <= 0) syserr("read stdin");
            if (write(pfdout[1], buf, nbytes) == -1) syserr("pfdout[1]");
        }

        if (FD_ISSET(pfdin[0], readFDs))
        {
            nbytes = read(pfdin[0], buf, sizeof(buf));
            if (nbytes == 0)
            {
                fprintf(stderr, "The server has hung up on us!\n");
                return;
            }
            else if (nbytes < 0)
            {
                syserr("read pfdin[0]");
            }
            if (write(1, buf, nbytes) == -1) syserr("write stdout");
            packbuf(buf, nbytes);
        }

        if (FD_ISSET(xServerFD, readFDs))
        {
            if (events(move, 1))
            {
                /* made a move. print it on the screen and send it to ICS */

                if (strcmp(move, "quit") == 0)
                {
                    done = 1;                  /* will exit soon */
                }
                fprintf(stdout, "%s\n", move);
                nbytes = strlen(move) + 1;
                move[nbytes-1] = '\n';
                if (write(pfdout[1], move, nbytes) == -1) syserr("write");
            }
        }
    }
    if (close(pfdin[0]) == -1) syserr("close");
    if (close(pfdout[1]) == -1) syserr("close");   /* telnet quits */

    /* in case telnet is still alive, kill it */

    kill(childpid, 9);
    return;
}
